#include <stdio.h>
#include "TROOT.h"
#include "TApplication.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TH2.h"
#include "TGraph.h"
#include "CTDResol.h"

//_____________________________________________________________________
//  -------------------------------
//  void CTDResol(p,tanl,sgk,sgkip)
//  -------------------------------
//  (Inputs)
//	Double_t p	:	momentum (GeV)
//	Double_t tanl	:	tan(lambda)
//  (Outputs)
//	Double_t &sgk	:	sigma_kappa w/o IP constraint (1/GeV)
//	Double_t &sgkip :	sigma_kappa w/  IP constraint (1/GeV)
//

void CTDResol(Double_t p, Double_t tanl, Double_t &sgk, Double_t &sgkip)
{
   // ------------------------------
   // Initialize material properties
   // ------------------------------

   Double_t x0vac   = 0.;
   Double_t x0be    = CTDLayer::kX0Be;
   Double_t x0air   = CTDLayer::kX0Air;
   Double_t x0si    = CTDLayer::kX0Si;
   Double_t x0cfrp  = CTDLayer::kX0CFRP;
   Double_t x0cdc   = CTDLayer::kX0CDC;

   // ------------------------------
   // Initialize detector properties
   // ------------------------------

   Double_t sgxdum  = 1.e1;	// dummy sigma_x (cm)
   Double_t bfield  = 2.;	// B field (T)

   Int_t    withvtx = 0;	// (0,1) = (w/o,w/) VTX
   Double_t sgxvtx  = 0.0004;	// sigma_x(VTX) (cm)

   Int_t    withimt = 0;	// (0,1) = (w/o,w/) IMT
   Double_t sgximt  = 0.0040;	// sigma_x(IMT) (cm)

   Int_t    nscdc   = 80;	// N_samples of CDC
   Double_t rincdc  = 45.;	// R_in(CDC)    (cm)
   Double_t routcdc = 230.;	// R_out(CDC)   (cm)
   Double_t sgxcdc  = 0.0085;	// sigma_x(CDC) (cm)
   Int_t    ns;
   Double_t elcdc;
   if (tanl <= 1.) {
      elcdc = routcdc - rincdc;
      ns    = nscdc;
   } else {
      elcdc = routcdc/tanl - rincdc;
      ns    = TMath::Nint(nscdc*elcdc/(routcdc-rincdc));
   }
   if (ns < 3) {
      cerr << "Not enough sampling points: ns = " << ns << endl;
   }

   // ------------------------------------
   // Initialize the tracker configuration
   // ------------------------------------
   // [1] How to define a layer
   //     Create a CTDLayer object with
   // 	    CTDLayer(name,Nsample,Rin,thickness,Bfield,X0,sgimax)
   //     wehere
   //		Char_t  *name		: name of the layer
   //		Int_t    Nsample	: No. samples in this layer
   //		Double_t Rin		: inner radius (cm)
   //		Double_t thickness	: thickness of the layer (cm)
   //		Double_t Bfield		: B field (T)
   //		Double_t X0		: radiation length (1/cm)
   //		Double_t sigmax		: resolution per sample (cm)
   //     Alternatively, one can use
   //	    CTDLayer(layer,Rin)
   //     where
   //		const CTDLayer &layer	: previous layer of the same kind
   //		Rin			: inner radius of the new layer (cm)
   //	  if some layer that only differs in Rin has already been defined.
   //	  The defined layer should be added to a CTDTracker object.
   //	  Once layers other than air are added, one can fill the
   //	  inter-layer dead gaps by air by calling
   //	    FillGapByAir(x0air)
   //	  member function of the CTDTracker class.
   //

   CTDTracker tracker;

   CTDLayer vac("Vacuum",0,0.0,2.00,bfield,x0vac,sgxdum); tracker.Add(&vac);
   CTDLayer bp("BM Pipe",0,2.0,0.05,bfield,x0be ,sgxdum); tracker.Add(&bp);
   CTDLayer vtx1("VTX",withvtx,2.4,0.03,bfield,x0si,sgxvtx); tracker.Add(&vtx1);
   CTDLayer vtx2(vtx1,3.6); tracker.Add(&vtx2);
   CTDLayer vtx3(vtx1,4.8); tracker.Add(&vtx3);
   CTDLayer vtx4(vtx1,6.0); tracker.Add(&vtx4);
   CTDLayer imt1("IMT",withimt,10.,0.03,bfield,x0si,sgximt); tracker.Add(&imt1);
   CTDLayer imt2(imt1,17.); tracker.Add(&imt2);
   CTDLayer imt3(imt1,24.); tracker.Add(&imt3);
   CTDLayer imt4(imt1,31.); tracker.Add(&imt4);
   CTDLayer imt5(imt1,38.); tracker.Add(&imt5);
   CTDLayer st("SUP",0,40.0,0.5,bfield,x0cfrp,sgxdum); tracker.Add(&st);
   CTDLayer cdc1("CDC",0,40.5,4.5,bfield,x0cdc,sgxdum); tracker.Add(&cdc1);
   CTDLayer cdc2("CDC",ns,45.0,elcdc,bfield,x0cdc,sgxcdc); tracker.Add(&cdc2);

   tracker.FillGapByAir(x0air);		// fills gaps by air

   // ------------------------------------
   // Print out the tracker configuration
   // ------------------------------------

   tracker.DebugPrint();

   // ------------------------------------
   // Calculate momentum resolution
   // ------------------------------------

   sgk   = tracker.GetSigmaKappa(p,tanl);	// sigma_kappa w/o IP
   sgkip = tracker.GetSigmaKappaIP(p,tanl);	// sigma_kappa w/  IP
}


//_____________________________________________________________________

TROOT root("CTDResol","Charged Track Detector Resolution Study");

int main(int argc, char **argv)
{
   // ---------------
   // Initialize ROOT
   // ---------------

   TApplication theApp("App", &argc, argv);

   // ---------------
   // Parse Arguments
   // ---------------

   TCanvas  *c1 = 0;
   Int_t     np, nc;
   Double_t *p, *costh;
   switch (argc) {
      case 1:
         np = nc = 1;
         p     = new Double_t [np];
         costh = new Double_t [nc];
         p    [0] = 50.;
         costh[0] = 0;
         break;
      case 2:
         np = nc = 1;
         p     = new Double_t [np];
         costh = new Double_t [nc];
         sscanf(argv[1],"%lg",&p[0]);
         costh[0] = 0;
         break;
      case 3:
         np = nc = 1;
         p     = new Double_t [np];
         costh = new Double_t [nc];
         sscanf(argv[1],"%lg",&p[0]);
         sscanf(argv[2],"%lg",&costh[0]);
         break;
      default:				// if >3 inputs, assume them being
         np = argc - 1;			// a list of mementa and create
         nc = 20;			// resolution plots
         p     = new Double_t [np];
         costh = new Double_t [nc];
         Int_t i;
         for (i=0; i<np; i++) sscanf(argv[i+1],"%lg",&p[i]);
         Double_t dy = 0.95/(nc-1);
         for (i=0; i<nc; i++) costh[i] = i*dy;
         break;
   }
   
   // ------------------------------------
   // Set up canvas for resolution plots
   // ------------------------------------
   
   if (nc > 1) {   
      c1 = new TCanvas("c1","Sigma_k",10,10,1000,500);
      c1->SetFillColor(42);
      c1->Divide(2,1);
      c1->cd(1);  gPad->SetFillColor(21);
		  gPad->SetGridx(); gPad->SetGridy(); gPad->SetLogy();
		  TH1F *frame1 = gPad->DrawFrame(0.,1.e-5,0.95,1.e-1);
		  frame1->SetTitle("w/o IP constraint");
		  frame1->SetXTitle("cos(theta)");
		  frame1->SetYTitle("sigma_k");
      c1->cd(2);  gPad->SetFillColor(21);
		  gPad->SetGridx(); gPad->SetGridy(); gPad->SetLogy();
		  TH1F *frame2 = gPad->DrawFrame(0.,1.e-5,0.95,1.e-1);
		  frame2->SetTitle("w/ IP constraint");
		  frame2->SetXTitle("cos(theta)");
		  frame2->SetYTitle("sigma_k");
   }
   
   // ------------------------------------
   // Calculate momentum resolution
   // ------------------------------------

   Double_t *sgk   = new Double_t [nc];
   Double_t *sgkip = new Double_t [nc];
   TGraph  *gr;
   
   cout.setf(ios::fixed);
   Int_t ip, ic;
   for (ip=0; ip < np; ip++) {
      cout << endl
           << "p(GeV)  cos(theta)  sigma_k(1/GeV)  sigma_k:IP(1/GeV)" << endl
           << "-----------------------------------------------------" << endl;
      for (ic=0; ic < nc; ic++) {
         Double_t tanl = costh[ic]/TMath::Sqrt((1-costh[ic])*(1+costh[ic]));
         CTDResol(p[ip],tanl,sgk[ic],sgkip[ic]);
         cout << setw(6)  << setprecision(1) << p[ip] 
              << setw(10) << setprecision(4) << costh[ic] 
              << setw(16) << setprecision(8) << sgk[ic]
              << setw(16) << setprecision(8) << sgkip[ic] << endl;
      }
      if (nc > 1) {
         c1->cd(1);
         gr = new TGraph(nc,costh,sgk);
         gr->SetFillColor(19);
         gr->SetLineColor(ip+1);
         gr->SetLineWidth(2);
         gr->Draw("C");
         gPad->Update(); 
         c1->cd(2);
         gr = new TGraph(nc,costh,sgkip);
         gr->SetFillColor(19);
         gr->SetLineColor(ip+1);
         gr->SetLineWidth(2);
         gr->Draw("C");
         gPad->Update(); 
      }
   }
   cerr.unsetf(ios::fixed);

   theApp.Run();

   return 0;
}
