//*************************************************************************
//* =================
//*  CTDTracker Class
//* =================
//*
//* (Description)
//*    Tracker class.
//* (Requires)
//*	class CTDLayer
//* (Provides)
//* 	class CTDTracker
//* (Update Recored)
//*    1999/11/08  K.Fujii	Original version.
//*
//*************************************************************************
//
#include "CTDTracker.h"
//_____________________________________________________________________
//  ----------
//  CTDTracker
//  ----------
//

ClassImp(CTDTracker)

//_____________________________________________________________________
void CTDTracker::DebugPrint(const Char_t *opt)
{
   cerr << "Number of layers = " << GetEntries() << endl;
   TIter next(this);
   CTDLayer *l;
   Int_t n = 0;
   while ((l = (CTDLayer *)next())) {
      cerr << setw(2) << ++n << " ";
      l->DebugPrint(opt);
   }
}

//_____________________________________________________________________
void CTDTracker::Sort(Int_t upto)
{
   TObjArray::Sort(upto);
   fSorted = kTRUE;
}

//_____________________________________________________________________
void CTDTracker::FillGapByAir(Double_t x0air)
{
   if (!fSorted) Sort();
   TIter next(this);
   CTDLayer *last = (CTDLayer *)Last();
   CTDLayer *l;
   Double_t rlast = 0.;
   while ((l = (CTDLayer *)next())) {
      if (rlast < l->fRin) {
         Add(new CTDLayer("Air",0,rlast,l->fRin-rlast,l->fB,x0air));
      }
      rlast = l->fRout;
      if (l == last) break;
   }
   fFilled = kTRUE;
   Sort();
}

//_____________________________________________________________________
TMatrix CTDTracker::MovePivot(Int_t in, const CTDLayer &l, const TMatrix &err)
{
   Double_t ptor  = CTDLayer::kCptor/l.fB;
   Double_t dy    = (l.fRin - l.fRout)*in;

   TMatrix dApdA(3,3);
   dApdA(0,0) = 1;
   dApdA(0,1) = -dy;
   dApdA(0,2) = dy*dy/2/ptor;
   dApdA(1,0) = 0;
   dApdA(1,1) = 1;
   dApdA(1,2) = -dy/ptor;
   dApdA(2,0) = 0;
   dApdA(2,1) = 0;
   dApdA(2,2) = 1;

   TMatrix Eb(dApdA);
   TMatrix dApdAt(TMatrix::kTransposed,dApdA);
   (Eb *= err) *= dApdAt;

   return Eb;
}

//_____________________________________________________________________
void CTDTracker::CalculateErrorMatrix(Double_t p, Double_t tanl)
{
   // ------------------------
   // E_full w/o IP constraint
   // ------------------------
   TIter next(this,kIterBackward);
   CTDLayer *l = (CTDLayer *)next();
   fError = l->GetEfull(p,tanl);
   while((l = (CTDLayer *)next())) {
      if (l->fNsample > 2) {		// Self-tracking layer
      	 TMatrix Efull(3,3);
      	 Efull = l->GetEfull(p,tanl);
      	 Efull.Invert();
      	 fError.Invert();
      	 fError += Efull;
      	 fError.Invert();
      } else if (l->fNsample > 1) {	// Invalid active layer
      	 cerr << "CTDTracker::CalculateErrorMatrix ...." << endl
              << "    Nsamle = " << l->fNsample << " is invalid" << endl
              << "    Divide the layer into two so that each layer" << endl
              << "    has a single sample point." << endl;
         return;
      } else {				// Other dead/active layer
         fError  = MovePivot(1,*l,fError);
         fError += l->GetDelEms(p,tanl);
         if (l->fNsample > 0) {
      	    fError.Invert();
      	    fError(0,0) = fError(0,0) + 1/TMath::Power(l->fSigmaX,2);
            fError.Invert();
         }
      }
   }
   // ------------------------
   // E_full w/ IP constraint
   // ------------------------
   TMatrix dAdB(2,3);
   dAdB(0,0) = 0;   dAdB(0,1) = 1;   dAdB(0,2) = 0;
   dAdB(1,0) = 0;   dAdB(1,1) = 0;   dAdB(1,2) = 1;
   TMatrix Einv(TMatrix::kInverted,fError);
   for (Int_t i=0; i<2; i++) {
      for (Int_t j=0; j<=i; j++) {
         fErrorIP(i,j) = 0;
         for (Int_t k=0; k<3; k++) {
            for (Int_t l=0; l<3; l++) {
               fErrorIP(i,j) = fErrorIP(i,j) + dAdB(i,k)*Einv(k,l)*dAdB(j,l);
            }
         }
         fErrorIP(j,i) = fErrorIP(i,j);
      }
   }
   fErrorIP.Invert();
   fErrorSet = kTRUE;
}

//_____________________________________________________________________
Double_t CTDTracker::GetSigmaKappa   (Double_t p, Double_t tanl)
{
   if (!fErrorSet) CalculateErrorMatrix(p,tanl);
   Double_t sg = TMath::Sqrt(fError(2,2));
   return sg;
}
//_____________________________________________________________________

Double_t CTDTracker::GetSigmaKappaIP (Double_t p, Double_t tanl)
{
   if (!fErrorSet) CalculateErrorMatrix(p,tanl);
   Double_t sg = TMath::Sqrt(fErrorIP(1,1));
   return sg;
}
